﻿//Game.as
package {
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.events.KeyboardEvent;
	import flash.events.Event;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	public class Game extends MovieClip {
		
		//Stage access
		private var _screen:Main;
		private var _time:Timer;
		private var _world:World;
		private var _background:Background;
		private var _player:Player;
		public var ui:UIManager;
		public const CW:uint = 8;
		public const SHOW_COLLBXS:Boolean = false;
		public const TOSS_DIRECT:Boolean = true;
		private const CWW:uint = CW + 83;
		private const COL1:uint = CW;
		private const COL2:uint = COL1 + CWW;
		private const COL3:uint = COL2 + CWW;
		private const COL4:uint = COL3 + CWW;
		private const COL5:uint = COL4 + CWW;
		private const COL6:uint = COL5 + CWW;
		private const CHILDSAC:Boolean = true;
		private const WORLDSTART:uint = 50;
		private const PARALLAXAMOUNT:uint = 5;
		
		//Sheet related
		private var _sheetStringArray:Array;
		private var _keyArray:Array;
		private var _fallingSheetArray:Array;
		private var _droppedSheetArray:Array;
		private var _stoppedSheetArray:Array;
		private var _sheetCount:uint;
		
		private var _windowArray:Array;
		private var _sheeterArray:Array;
		private var _childArray:Array;
		private var _nannyArray:Array;
		
		//Bool for sheets movement (fall/drop)
		private var _sheetFall:Boolean;
		
		//For screen/world movemnet
		private var _worldHighMove:uint;
		private var _worldLowMove:uint;
		
		//For winning
		private var _won:Boolean;
		private var _winLine:uint;
		
		//For pausing
		private var _paused:Boolean;
		private var _pauseScn:PauseScn;
		
		private var _warning:Boolean;
		
		//For XML Level Loading
		//Tutorial: http://www.kirupa.com/developer/flashcs3/using_xml_as3_pg2.htm
		private var _xmlLoader:URLLoader;
		private var _levelName:String;
		private var _levelFile:String;
		
		public function Game(value:Main, level:String) {
			//First, let's setup our screen/stage
			//access so we can use it later...
			_screen = value;
			_levelFile = level;
			
			_background = new Background();
			addChild(_background);
			
			//Make the world separate so we can move it when the player moves
			_world = new World(this);
			_world.x = (_screen.stage.stageWidth / 2) - (_world.width / 2);
			_world.y = WORLDSTART;
			addChild(_world);
			
			//Sets the boundaries for when the world/screen
			//is supposed to shift/move
			_worldHighMove = _screen.stage.stageHeight / 1.5;
			_worldLowMove = _screen.stage.stageHeight - _worldHighMove;
			
			//trace(_worldHighMove, _worldLowMove);
			
			_winLine = _world.height - 70;
			
			//Now, let's add the UI...
			ui = new UIManager(this);
			
			//Add the pause screen (we're not paused by default)
			_pauseScn = new PauseScn(this, getMain, _levelFile);
			addChild(_pauseScn);
			
			//Sheet arrays for storing the strings of sheets
			//we can climb on and the sheets that are falling
			//dropping, or that are stopped on windows
			_sheetStringArray = new Array();
			_fallingSheetArray = new Array();
			_droppedSheetArray = new Array();
			_stoppedSheetArray = new Array();
			_windowArray = new Array();
			
			_sheeterArray = new Array();
			_childArray = new Array();
			_nannyArray = new Array();
			
			//Main timer
			_time = new Timer(10);
			_time.addEventListener(TimerEvent.TIMER, update, false, 0, true);

			//Now, we need store player input...
			//All keys are up by default (false)
			_keyArray = new Array();
			
			//XML level loading
			_xmlLoader = new URLLoader();
			_xmlLoader.addEventListener(Event.COMPLETE, loadLevel);
			_xmlLoader.load(new URLRequest("./"+level));
		} //Constructor
		
		private function loadLevel(e:Event):void{
			var xmlData = new XMLList(e.target.data);
			var error:Boolean;
			var type:String;
			var column:uint;
			var height:uint;
			var pSheets:uint;
			var pChildren:uint;
			_levelName = xmlData.attribute("name");
			ui.setLevelName = _levelName;
			var windowList:XMLList = xmlData.Window;
			if ((windowList.length() > 0) &&
					(windowList[0].text() != '')){
				for (var i = 0; i < windowList.length(); i++) {
					//TODO: Ignore missing window vars
					//trace(i+" Type: "+windowList[i].Type.length());
					//trace(i+" Type: "+windowList[i].Type.text());
					//trace(i+" Column: "+windowList[i].Column.length());
					//trace(i+" Column: "+windowList[i].Column.text());
					//trace(i+" Height: "+windowList[i].Height.length());
					//trace(i+" Height: "+windowList[i].Height.text());
					if ((windowList[i].Type.length() > 0) &&
					(windowList[i].Type.text() != '') &&
					(windowList[i].Column.length() > 0) &&
					(windowList[i].Column.text() != '') &&
					(windowList[i].Height.length() > 0) &&
					(windowList[i].Height.text() != '')) {
						type = windowList[i].Type.text();
						column = windowList[i].Column.text();
						height = windowList[i].Height.text();
						switch(column){
							case 1:	column = COL1;	break;
							case 2:	column = COL2;	break;
							case 3:	column = COL3;	break;
							case 4:	column = COL4;	break;
							case 5:	column = COL5;	break;
							case 6:	column = COL6;	break;
						}
						_world.addWindow(type, column, height);
					}else{
						trace("Window info error!");
					} //Check window's vars
				} //Load windows for
				
				//Sort the array by the window appearance (x,y)
				_windowArray.sortOn(["y", "x"], Array.NUMERIC);
				
				for (i = 0; i < _nannyArray.length; i++) {
					_nannyArray[i].findWindow();
				}
				
				//All windows are now set up!
				//Time to set up the player
				
				var playerStats:XMLList = xmlData.Player;
				
				if ((playerStats.length() > 0) &&
					(playerStats[0].text() != '')){
					if ((playerStats[0].Sheets.length() > 0) &&
						(playerStats[0].Sheets.text()[0] != '')){
						pSheets = playerStats[0].Sheets.text()[0];
						//trace(pSheets);
					}else {
						//Match with defaults
						pSheets = 4;
					}
					if ((playerStats[0].Children.length() > 0) &&
						(playerStats[0].Children.text()[0] != '')){
						pChildren = playerStats[0].Children.text()[0];
						//trace(pChildren);
					}else {
						//Match with defaults
						pChildren = 0;
					}
					_world.addPlayer(pSheets,pChildren);
				}else{
					_world.addPlayer();
				}
				
				_player = _world.getPlayer;
				
				//Start!
				_time.start();
			}else {
				trace("No windows found!");
			}
		}
		
/*********************UPDATE FUNCTION*********************/
		private function update (Timer:TimerEvent):void{
			if (!_paused){
				//Check for warning
				if (_warning){
					ui.moveWarningBird();
				}
				//Check for sheets falling/dropped
				if (_sheetFall){
					moveFallingSheets(_fallingSheetArray);
					moveFallingSheets(_droppedSheetArray);
				}
				//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
				//Yay for reuse! ^.\
				//Check for movement (if not dead/blocked)
				
				var hitWindowsill:Window = _player.hitWindow("windowsill");
				var hitWindow:Window = _player.hitWindow("window");
				var hitString:SheetString = _player.hitString();
				
				if ((!hitWindowsill) &&
					(!hitWindow) &&
					(!hitString)){
						_player.falling = true;
						_player.dead = true;
				}
				
				if ((hitWindowsill) || (hitString)){
					_player.falling = false;
					
				}else if (hitWindow){
					_player.falling = true;
				}
				
				if ((!_player.dead) && (_player.ableToMove)) {
					keyCheck(hitString);
					
					var theWindow:Window;
					var amount:uint;
					var theSheeter:DisplayObject = _player.hitSheeter();
					var theChild:Child = _player.hitChild();
					var theNanny:Nanny = _player.hitNanny();
					var nannyArea:Nanny = _player.hitNannyArea();
					var droppedSheet:DroppedSheet = _player.hitDroppedSheet();
					var tempArray:Array;
					var number:uint;
					
					if (theSheeter != null) {
						number = _sheeterArray.indexOf(theSheeter);
						tempArray = theSheeter.name.split("-");
						amount = tempArray.shift();
						//theWindow = getWindowArray[tempArray.shift()];
						theWindow = Window(theSheeter.parent);
						if (!theWindow.covered){
							_player.sheetAmount += amount;
							theWindow.removeChild(theSheeter);
							_sheeterArray.splice(number, 1);
							_world.addChild(new Notif(getWorld, amount));
						}
					}
					
					if (theChild != null) {
						number = _childArray.indexOf(theChild);
						tempArray = theChild.name.split("-");
						//theWindow = getWindowArray[tempArray.splice(1, 1)];
						theWindow = Window(theChild.parent);
						if (!theWindow.covered){
							_player.childAmount++;
							theWindow.removeChild(theChild);
							_childArray.splice(number, 1);
							_world.addChild(new Notif(getWorld, "+c"));
						}
					}
					
					if ((theNanny != null) &&
						(!theNanny.getParentWindow.covered)){
						GameOver();
					}
					
					if (nannyArea != null){
						dropSheet(nannyArea);
					}
					
					if (droppedSheet != null){
						if ((!CHILDSAC) ||
							((CHILDSAC) && (_player.childAmount == 0))){
							_player.ableToMove = false;
							GameOver();
						}else {
							//Sacrifice a child
							_player.ableToMove = true;
							_player.childAmount--;
							//Bring player to front
							_world.addChild(_player);
							_world.addChild(new Notif(getWorld, "-c"));
						}
						
						//Decay
						hitWindow.covered = true;
						tempArray = _droppedSheetArray.splice(droppedSheet, 1);
						tempArray[0].name = "s" + _stoppedSheetArray.length;
						_stoppedSheetArray.push(tempArray.pop());
						tempArray = null;
						_stoppedSheetArray[_stoppedSheetArray.length - 1].startTimer();
					}
					
					if (_player.y >= _winLine){
						YouWin();
					}
				}else{
					//We're dead
					if ((_player.y >=
					((( -1 * _world.y) + _screen.stage.stageHeight) + 150))){
						GameOver();
					}
				}
			} //Pause check
		} //update
/*********************UPDATE FUNCTION*********************/
		
/*********************FUNCTIONS FOR KEYS*********************/
		public function timerKeyDown(e:KeyboardEvent):void{_keyArray[e.keyCode] = true;}
		public function timerKeyUp(e:KeyboardEvent):void{
			_keyArray[e.keyCode] = false;
		
			//Spacebar pauses/unpauses
			if (e.keyCode == 32){gamePaused = !gamePaused;}
		} //timerKeyUp
		
		private function keyCheck(value:SheetString):void {
			//TURNING, MOVING LEFT/RIGHT
			/* A, LEFT */	if (_keyArray[65] || _keyArray[37]){moveLeft(_player.moveAmount);}
			/* D, RIGHT */	if (_keyArray[68] || _keyArray[39]){moveRight(_player.moveAmount);}
			
			//CLIMBING, MOVING UP/DOWN
			//Climbing moves the player as well as the world itself
			if (value){
			/* W, UP */		if (_keyArray[87] || _keyArray[38]){moveUp(_player.moveAmount);}
			/* S, DOWN */	if (_keyArray[83] || _keyArray[40]){moveDown(_player.moveAmount);}
			}
		} //keyCheck
/*********************FUNCTIONS FOR KEYS*********************/

		public function dropSheet(theNanny:Nanny = null) {
			if ((theNanny != null) || 
				((_player.sheetAmount > 0) &&
				(!_player.dead) &&
				(!_player.falling) && 
				(_player.hitWindow("windowsill"))/* &&
				((_player.hitWindow("left")) ||
				(_player.hitWindow("right")))*/)){
				//Checked here and not on the Drop button itself so we
				//don't have to pass the drop button another arg
				var _dSheet:DroppedSheet = new DroppedSheet(this, theNanny);
				if (theNanny == null){
					_world.addChildAt(_dSheet, _world.getChildIndex(_player));
				}else{
					_world.addChild(_dSheet);
				}
				if(!_sheetFall){_sheetFall = true;}
			}
		} //dropSheet
		
		private function moveFallingSheets(theArray:Array):void {
			if (theArray.length < 1){return;}
			var _fallSpeed:uint;
			var _fallBuff:uint;
			var _stoppedSheet:DroppedSheet;
			if (theArray[0].name.charAt() == "f"){
				_fallSpeed = 3;		//Falling sheet
				_fallBuff = 50;
			}else {
				_fallSpeed = 3;		//Dropped sheet
				_fallBuff = _screen.stage.stageHeight;
			}
			//Go through the falling/dropping sheet array
			for (var i:uint = 0; i < theArray.length; i++) {
				var dontMove:Boolean = false;
				//If we are a dropping sheet...
				if (theArray[i].name.charAt() == "d") {
					//Have the dropping sheet check for window hits
					//If we hit a window...
					if (theArray[i].checkWindColl){
						//Disable a window's ability
						theArray[i].getWindow.covered = true;
						//Take the sheet off the array and put
						//it into the other (stopped dropped
						//sheets) array
						var tempArray:Array = theArray.splice(theArray[i], 1);
						tempArray[0].name = "s" + _stoppedSheetArray.length;
						_stoppedSheetArray.push(tempArray.pop());
						tempArray = null;
						//Make sheet appear behind player if it
						//hits a windowsill and not them
						_world.addChildAt(_stoppedSheetArray[_stoppedSheetArray.length -1], _world.getChildIndex(_player) - 1);
						_stoppedSheetArray[_stoppedSheetArray.length - 1].startTimer();
						//Set it so we don't move the sheet
						//since we took it of the array
						dontMove = true;
					}
				}
				if (!dontMove) {
					//Move the falling/dropped sheet
					theArray[i].y += _fallSpeed;
				}
			}
			
			//We're only removing a sheet if it's below the visible
			//part of the stage. (and if we actually have sheets to
			//remove since they could be removed above)
			
			//We find this by multiplying what's above the stage
			//(_world.y) by negative 1 to make it positive, thus becoming
			//the top of the visible stage and then add it to the stage
			//height to get the point at which the visible stage ends
			
			//...
			
			//Whew!
			if (theArray.length > 0){
				if ((theArray[theArray.length-1].y) >
					(((-1 * _world.y) + _screen.stage.stageHeight) + _fallBuff)){
					//The last sheet in the array will always
					//be the first to get off the stage
					subSheet(theArray);
				}
			}
			
			if (_fallingSheetArray.length == 0 && _droppedSheetArray.length == 0){
				//No more falling sheets
				_sheetFall = false;
			}
		} //moveFallingSheets

		public function GameOver():void{
			_time.stop();
			_screen.gameOver(_levelFile);
		} //GameOver
		
		public function YouWin():void {
			_time.stop();
			getMain.getBW.data.won.push(_levelFile);
			_screen.win(_levelFile);
		} //YouWin

		public function subSheet(theArray:Array, theString:SheetString = null):void {
			
			//This method is run a few times:
			//1.) When a sheet (falling or dropped) is out of
			//		sight on the stage
			//2.) When the player chooses to remove a sheet from
			//		the game
			//3.) When a dropped sheet is "done" on a windowsill
			
			//If we're a stopped sheet
			if (theArray[0].name.charAt() == "s"){
				//Remove it
				var theSheet:DroppedSheet = theArray.shift();
				if (theSheet.getNanny != null){
					if (theSheet == theSheet.getNanny.firedSheet){
						theSheet.getNanny.firedSheet = null;
						theSheet.getNanny.fired = false;
						if (!_player.ableToMove){
							_player.ableToMove = true;
						}
					}
				}
				_world.removeChild(theSheet);
				theSheet.getWindow.covered = false;
				theSheet = null;
			}else{
				//Gets the last sheet
				var lastSheet:DisplayObject =
					theArray[theArray.length - 1];
				
				//Double check to make sure that the player isn't on top of
				//a sheet he/she is about to remove
				// --or--
				//If the player is above the sheet about to be removed
				//AND they can move, then that means that they're not
				//frozen so they are still alive
				if (!_player.dead){
					if (_player.y < lastSheet.y){
						//We are ALIVE!
						
						//If we're removing the sheet to be chosen to be dropped
						//then stop the untie timer because we're untying it early
						if (((theArray.length - 1) == theArray.chosenSheet) &&
							theString != null) {
							theString.updateTimers();
						}
						//Give the sheet back to the player ONLY if the
						//sheet is not part of the _fallSheetArray or
						//_droppedSheetArray. Sheets in these arrays start
						//with f or d, so it's easy to find them
						if (lastSheet.name.charAt() != "f" &&
							lastSheet.name.charAt() != "d"){
							_player.sheetAmount++;
						}
						//Remove it
						theArray.pop();
						_world.removeChild(lastSheet);
					}
				}else {
					//We're dead, so we're just going to
					//remove falling/dropped sheets, and NOT hanging
					//sheets
					if (lastSheet.name.charAt() == "f" ||
						lastSheet.name.charAt() == "d"){
						theArray.pop();
						_world.removeChild(lastSheet);
					}
				}
			} //stopped
		} //subSheet
		
		public function removeDropped():void{
			subSheet(_stoppedSheetArray);
		}
		
		private function moveUp(amount:uint):void {
			if (_player.hitSheet() != null){
				_player.y -= amount;
				//Move world down only if we hit the line
				if (_player.y < _worldHighMove){
					//_world.y would be negative or less than the WORLDSTART
					if (_world.y <= WORLDSTART){
						_world.y += amount;
						if (_background.y <= 0){
						_background.y += (amount / PARALLAXAMOUNT);
						}else/* if (_background.y > 0)*/{
							_background.y = 0;
						}
					}
					
					//Since we're moving our world down,
					//we need to also move the trigger points
					//for moving it again up
					_worldHighMove -= amount;
					_worldLowMove -= amount;
				}
			}
		} //moveUp
		
		public function moveDown(amount:uint):void {
			if (!_player.dead){
				//If we're hitting a window while falling...
				//Checked here again because of falling, hence not
				//passing in a var for the hit window
				var theWindow:Window = _player.hitWindow("window");
				if (theWindow && _player.falling){
					//Then get the location of the windowsill for thw window.
					//This is a Number because a windowsill could be in a very
					//exact spot.
					var theSill:Number = theWindow.y + theWindow.sill.y;
					//If where we're going to be is NOT going to be
					//inside the windowsill, then...
					if ((_player.y + amount) <= theSill) {
						//We're fine, move down
						_player.y += amount;
					}else {
						//..else, it is, so just pop us on top
						//of the windowsill
						_player.y = theSill;
					}
				}else {
					//For normal climbing down
					_player.y += amount;
				}
				//Move world up, only if we cross the line
				if (_player.y > _worldLowMove){
					
					//The world y would be negative, since the
					//registration point is in the upper left
					//hand corner, so it would be above the stage.
					//We minus what currently is on the stage from
					//its total height to get the y.
					// Example:
					//(-1)(600 - 400) = -200
					
					//Brainfart: the if here is needed to check for
					//the bottom of the world.
					
					if (
						(( -1 * (_world.height + WORLDSTART)) +
						_screen.stage.stageHeight) <
						(_world.y - WORLDSTART - amount)
					){
						
						//Move the world up
						_world.y -= amount;
						//Move the background up
						if (_background.y >= (-1 * (_background.height - _screen.stage.stageHeight))){
							_background.y -= (amount / PARALLAXAMOUNT);
						}else/* if (_background.y < (-1 * (_background.height - _screen.stage.stageHeight)))*/{
							_background.y = (-1 * (_background.height - _screen.stage.stageHeight));
						}
						
						//Since we're moving our world up,
						//we need to also move the trigger points
						//for moving it down again
						_worldHighMove += amount;
						_worldLowMove += amount;
					}else{ //...else, we're at the bottom...make sure it doesn't move
						//_world.y = (( -1 * (_world.height/* + WORLDSTART*/)) +
						//_screen.stage.stageHeight);
					}
				}
			}else {
				//We're dead, so just fall
				_player.y += amount;
			}
		} //moveDown
		
		private function moveLeft(amount:uint):void{
			_player.scaleX = -1;
			var theWindow:Window = _player.hitWindow("windowsill");
			if (((theWindow != null) &&
				(_player.y <= theWindow.y + theWindow.sill.y)) ||
				(theWindow == null)){
				_player.x -= amount;
			}
		} //moveLeft
		
		private function moveRight(amount:uint):void {
			_player.scaleX = 1;
			var theWindow:Window = _player.hitWindow("windowsill");
			if (((theWindow != null) &&
				(_player.y <= theWindow.y + theWindow.sill.y)) ||
				(theWindow == null)){
				_player.x += amount;
			}
		} //moveRight
		
/*********************GET/SET FUNCTIONS BELOW*********************/



/*********************GET WORLD*********************/
		public function get getWorld():World {
			return _world;}
/*********************GET WORLD*********************/

/*********************GET MAIN*********************/
		public function get getMain():Main {
			return _screen;}
/*********************GET MAIN*********************/

/*********************GET SHEET STRING ARRAY*********************/
		public function get getStringSheetArray():Array {
			return _sheetStringArray;}
/*********************GET SHEET STRING ARRAY*********************/

/*********************GET WINDOW ARRAY*********************/
		public function get getWindowArray():Array {
			return _windowArray;}
/*********************GET WINDOW ARRAY*********************/

/*********************GET/SET DROPPED SHEET ARRAY*********************/
		public function get droppedSheetsArray():Array {
			return _droppedSheetArray; }

		public function set droppedSheetsArray(value:Array):void {
			_droppedSheetArray = value;}
/*********************GET/SET DROPPED SHEET ARRAY*********************/

/*********************GET/SET GAME PAUSED*********************/
		public function set gamePaused(value:Boolean):void{
			var i:uint;
			_paused = value;
			if (_paused){
				ui.enableButtons(false);
				//Stop ALL the timers
				for (i = 0; i < _sheetStringArray.length; i++){
					_sheetStringArray[i].stopTimers();}
				for (i = 0; i < _stoppedSheetArray.length; i++){
					_stoppedSheetArray[i].stopTimer();}
				_pauseScn.visible = true;
			}else{
				ui.enableButtons(true);
				//Don't start the chosen/untie sheet timers if a
				//sheet actually hasn't been chosen yet
				//TODO: start restarts timer instead of pauses it
				//Start ALL the timers (if applicable)
				for (i = 0; i < _sheetStringArray.length; i++){
					_sheetStringArray[i].startTimers();}
				for (i = 0; i < _stoppedSheetArray.length; i++){
					_stoppedSheetArray[i].startTimer();}
				_pauseScn.visible = false;
			} //Paused if
		}
		
		public function get gamePaused():Boolean{
			return _paused;}
/*********************GET/SET GAME PAUSED*********************/
		
		public function set warning(value:Boolean):void {
			_warning = value;}
		
		public function get warning():Boolean{
			return _warning;}
		
		public function set sheetFall(value:Boolean):void{
			_sheetFall = value;}
		
		public function get sheetFall():Boolean{
			return _sheetFall;}
		
		public function set fallingSheetArray(value:Array):void {
			_fallingSheetArray = value;}
		
		public function get fallingSheetArray():Array {
			return _fallingSheetArray; }
		
		public function set sheeterArray(value:Array):void {
			_sheeterArray = value;}
		
		public function get sheeterArray():Array{
			return _sheeterArray; }
		
		public function set childArray(value:Array):void {
			_childArray = value;}
		
		public function get childArray():Array{
			return _childArray; }
		
		public function set nannyArray(value:Array):void {
			_nannyArray = value;}
		
		public function get nannyArray():Array{
			return _nannyArray; }
		} //Game class
} //Package