﻿//SheetString.as
package{
	import flash.display.MovieClip;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import flash.display.DisplayObject;
	public class SheetString extends MovieClip{
		
		//Edit the timers if needed
		private const cstime = 3500; //in ms
		private const uttime = 1500; //in ms
		
		private var _chooseSheetTimer:Timer;
		private var _untieSheetTimer:Timer;
		private var _sheetArray:Array;
		private var _fallingSheetArray:Array;
		private var _chosenSheet:uint;
		private var _minRand:uint;
		private var _world:World;
		private var _game:Game;
		private var _player:Player;
		private var _warning:Boolean;
		private var _sheetFall:Boolean;
		
		public function SheetString(value:World) {
			_world = value;
			_game = _world.getGame;
			_player = _world.getPlayer;
			_warning = _world.getGame.warning;
			_sheetFall = _world.getGame.sheetFall;
			
			_minRand = 0;
			
			//New sheet chosen every...
			_chooseSheetTimer = new Timer(cstime);
			_chooseSheetTimer.addEventListener(TimerEvent.TIMER, chooseSheet, false, 0, true);
			//We'll start the timer later once more sheets are added
			
			//Sheet partially untied every...
			_untieSheetTimer = new Timer(uttime);
			_untieSheetTimer.addEventListener(TimerEvent.TIMER, untieSheet, false, 0, true);
			//We'll start the timer later once more sheets are added
			
			//No sheets should be chosen to be canidates for falling,
			//so let's make it a high number that will act as our
			//'null' value (uint doesn't have null)
			_chosenSheet = 2000;
			
			_sheetArray = new Array();
		} //Constructor
		
		public function updateTimers():void {
			if (_sheetArray.length > 0) {
				//If there is a sheet, then start picking one
				//to loosen after a certain amount of time
				//if it's not already running
				if (!_chooseSheetTimer.running){_chooseSheetTimer.start();}
			}
		} //updateTimers
		
		private function chooseSheet(Timer:TimerEvent):void {
			//Called when it's time for the game to choose a
			//sheet to untie
			
			//Don't generate something if we've already
			//chosen a sheet to affect
			//2000 is the 'null' number.
			if ((2000 == _chosenSheet) &&
				(!_untieSheetTimer.running)){
				_chooseSheetTimer.stop();
				
				var max:uint = _sheetArray.length - 1;
				_chosenSheet = Math.ceil(Math.random() * (max - _minRand + 1))
				+ (_minRand - 1);
				_untieSheetTimer.start();
				//A falling sheet must be out of sight...warn the player
				if (( -1 * (_world.y)) > _sheetArray[_chosenSheet].y){
					_warning = true;
				}
			}
		} //chooseSheets
		
		private function untieSheet(Timer:TimerEvent):void {
			_sheetArray[_chosenSheet].alpha -= 0.2;
			if (_sheetArray[_chosenSheet].alpha <= 0.2) {
				_untieSheetTimer.stop();
				//Free fallin'...
				
				//Check to see if we're on (or below) the
				//sheet when it's fully untied
				
				//If we're hitting this specific sheet string...
				if (_player.hitString() == this) {
					//then see if we're below the falling sheet
					if (_player.y >= _sheetArray[_chosenSheet].y){
						_player.dead = true;
					}
				}
				
				var _tempArray:Array = _sheetArray.splice(_chosenSheet);
				
				//Rename all the sheets
				for (var i = 0; i < _tempArray.length; i++){
					_tempArray[i].name = "fsheet" +
					(_world.getGame.fallingSheetArray.length + i);
				}
				
				//Add the new elements to the gloabal falling sheet array
				_world.getGame.fallingSheetArray =
				_world.getGame.fallingSheetArray.concat(_tempArray);
				
				//Start falling, if not already
				if(!_sheetFall){_world.getGame.sheetFall = true;}
				
				//Loop through all of the windows
				for (i = 0; i < _game.getWindowArray.length; i++) {
					//...and see if our sheetstring is on this window.
					//If it is, check to see if it's empty yet, and if
					//it is, take the flag off of that part of the window
					//so we can place sheets there again
					if (_game.getWindowArray[i].lString == this &&
						_sheetArray.length == 0){
						_game.getWindowArray[i].lString = null;
					}else if (_game.getWindowArray[i].rString == this &&
						_sheetArray.length == 0){
						_game.getWindowArray[i].rString = null;
					}
				}
				
				//Time to pick another sheet
				//Not resetting _sheetFall yet
				//until all the sheets are gone
				_chosenSheet = 2000; //Null
				stopTimers();
				updateTimers();
			}
		} //untieSheet
		
		public function stopTimers():void{
			_chooseSheetTimer.stop();
			_untieSheetTimer.stop();
		} //stopTimers
		
		public function startTimers():void{
			if (2000 != _chosenSheet) {
				_chooseSheetTimer.start();
				_untieSheetTimer.start();
			}
		} //startTimers
		
		public function get getSheetArray():Array{return _sheetArray;}
	} //SheetString class
} //Package